#!/usr/bin/env python
#
# PyODConverter (Python OpenDocument Converter) v1.0.0 - 2008-05-05
#
# This script converts a document from one office format to another by
# connecting to an OpenOffice.org instance via Python-UNO bridge.
#
# Copyright (C) 2008 Mirko Nasato <mirko@artofsolving.com>
#                    Matthew Holloway <matthew@holloway.co.nz>
# Licensed under the GNU LGPL v2.1 - http://www.gnu.org/licenses/lgpl-2.1.html
# - or any later version.
#

DEFAULT_OPENOFFICE_PORT = 8100

from os.path import abspath
from os.path import isfile
from os.path import splitext
import sys
from StringIO import StringIO

import uno
import unohelper
from com.sun.star.beans import PropertyValue
from com.sun.star.uno import Exception as UnoException
from com.sun.star.connection import NoConnectException, ConnectionSetupException
from com.sun.star.lang import IllegalArgumentException
from com.sun.star.io import XOutputStream
from com.sun.star.io import IOException

class DocumentConversionException(Exception):

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message

class OutputStreamWrapper(unohelper.Base, XOutputStream):
    """ Minimal Implementation of XOutputStream """
    def __init__(self, debug=True):
        self.debug = debug
        self.data = StringIO()
        self.position = 0
        if self.debug:
            sys.stderr.write("__init__ OutputStreamWrapper.\n")

    def writeBytes(self, bytes):
        if self.debug:
            sys.stderr.write("writeBytes %i bytes.\n" % len(bytes.value))
        self.data.write(bytes.value)
        self.position += len(bytes.value)

    def close(self):
        if self.debug:
            sys.stderr.write("Closing output. %i bytes written.\n" % self.position)
        self.data.close()

    def flush(self):
        if self.debug:
            sys.stderr.write("Flushing output.\n")
        pass
    def closeOutput(self):
        if self.debug:
            sys.stderr.write("Closing output.\n")
        pass

class DocumentConverter:
   
    def __init__(self, host='localhost',port=DEFAULT_OPENOFFICE_PORT):
        self.localContext = uno.getComponentContext()
        self.serviceManager = self.localContext.ServiceManager
        resolver = self.serviceManager.createInstanceWithContext("com.sun.star.bridge.UnoUrlResolver", self.localContext)
        try:
            context = resolver.resolve("uno:socket,host=%s,port=%s;urp;StarOffice.ComponentContext" % (host, port))
        except IllegalArgumentException, exception:
            raise DocumentConversionException, "The url is invalid (%s)" % exception
        except NoConnectException, exception:
            raise DocumentConversionException, "Failed to connect to OpenOffice.org on port %s. %s" % (port, exception)
        except ConnectionSetupException, exception:
            raise DocumentConversionException, "Not possible to accept on a local resource (%s)" % exception
        self.desktop = context.ServiceManager.createInstanceWithContext("com.sun.star.frame.Desktop", context)

    def convertByStream(self, stdinBytes):
        inputStream = self.serviceManager.createInstanceWithContext("com.sun.star.io.SequenceInputStream", self.localContext)
        inputStream.initialize((uno.ByteSequence(stdinBytes),))

        document = self.desktop.loadComponentFromURL('private:stream', "_blank", 8, self._toProperties(
            InputStream = inputStream
            ))

        if not document:
            raise DocumentConversionException, "Error making document"
        try:
            document.refresh()
        except AttributeError, e:
            pass
        outputStream = OutputStreamWrapper(False)
        try:
            document.storeToURL('private:stream', self._toProperties(
                OutputStream = outputStream,
                FilterName = "writer_pdf_Export"))
        except IOException, e:
            print ("IOException during conversion: %s - %s" % (e.ErrCode, e.Message))

        document.close(True)
        openDocumentBytes = outputStream.data.getvalue()
        outputStream.close()
        return openDocumentBytes

    def convertByPath(self, inputFile, outputFile):
        inputUrl = self._toFileUrl(inputFile)
        outputUrl = self._toFileUrl(outputFile)
        document = self.desktop.loadComponentFromURL(inputUrl, "_blank", 8, self._toProperties(Hidden=True))
        try:
            document.refresh()
        except AttributeError:
            pass
        try:
            document.storeToURL(outputUrl, self._toProperties(FilterName="writer_pdf_Export"))
        finally:
            document.close(True)

    def _toFileUrl(self, path):
        return uno.systemPathToFileUrl(abspath(path))

    def _toProperties(self, **args):
        props = []
        for key in args:
            prop = PropertyValue()
            prop.Name = key
            prop.Value = args[key]
            props.append(prop)
        return tuple(props)

